@@ -23,6 +23,8 @@
 #include <glib/glib.h>
 #include <gio/gio.h>
 #include <stdlib.h>
+#include <unistd.h>
+#include <sys/types.h>
 #include <string.h>
 #include <sys/stat.h>
 
@@ -100,6 +102,7 @@ static const struct StructureItem sample_struct[] = {
 	{"lost_symlink",		"nowhere",	G_FILE_TYPE_SYMBOLIC_LINK, G_FILE_CREATE_NONE, 0, 0, TEST_COPY | TEST_DELETE_NORMAL | TEST_OPEN | TEST_INVALID_SYMLINK},
   };
 
+static gboolean test_suite;
 static gboolean write_test;
 static gboolean verbose;
 static gboolean posix_compat;
@@ -181,7 +184,7 @@ test_create_structure (gconstpointer test_data)
   GError *error;
   GFileOutputStream *outs;
   GDataOutputStream *outds;
-  int i;
+  guint i;
   struct StructureItem item;
 
   g_assert (test_data != NULL);
@@ -220,6 +223,10 @@ test_create_structure (gconstpointer test_data)
 	       item.link_to);
 	  child = create_symlink (root, item.filename, item.link_to);
 	  break;
+        case G_FILE_TYPE_UNKNOWN:
+        case G_FILE_TYPE_SPECIAL:
+        case G_FILE_TYPE_SHORTCUT:
+        case G_FILE_TYPE_MOUNTABLE:
 	default:
 	  break;
 	}
@@ -385,7 +392,7 @@ test_initial_structure (gconstpointer test_data)
   gboolean res;
   GError *error;
   GFileInputStream *ins;
-  int i;
+  guint i;
   GFileInfo *info;
   guint32 size;
   guchar *buffer;
@@ -458,8 +465,8 @@ test_initial_structure (gconstpointer test_data)
 			     PATTERN_FILE_SIZE, NULL, &error);
       g_assert_no_error (error);
       total_read += read;
-      log ("      read %d bytes, total = %d of %d.\n", (int) read,
-	   (int) total_read, PATTERN_FILE_SIZE);
+      log ("      read %"G_GSSIZE_FORMAT" bytes, total = %"G_GSSIZE_FORMAT" of %d.\n",
+	   read, total_read, PATTERN_FILE_SIZE);
     }
   g_assert_cmpint (total_read, ==, PATTERN_FILE_SIZE);
 
@@ -486,7 +493,7 @@ traverse_recurse_dirs (GFile * parent, GFile * root)
   GFileInfo *info;
   GFile *descend;
   char *relative_path;
-  int i;
+  guint i;
   gboolean found;
 
   g_assert (root != NULL);
@@ -571,7 +578,7 @@ test_enumerate (gconstpointer test_data)
   GError *error;
   GFileEnumerator *enumerator;
   GFileInfo *info;
-  int i;
+  guint i;
   struct StructureItem item;
 
 
@@ -711,8 +718,17 @@ do_copy_move (GFile * root, struct StructureItem item, const char *target_dir,
   else if (((item.extra_flags & TEST_NO_ACCESS) == TEST_NO_ACCESS) ||
 	   (extra_flags == TEST_NO_ACCESS))
     {
-      g_assert_cmpint (res, ==, FALSE);
-      g_assert_error (error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED);
+      /* This works for root, see bug #552912 */
+      if (test_suite && getuid () == 0)
+	{
+	  g_assert_cmpint (res, ==, TRUE);
+	  g_assert_no_error (error);
+	}
+      else
+	{
+	  g_assert_cmpint (res, ==, FALSE);
+	  g_assert_error (error, G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED);
+	}
     }
   /*  no error should be found, all exceptions defined above  */
   else
@@ -735,7 +751,7 @@ test_copy_move (gconstpointer test_data)
 {
   GFile *root;
   gboolean res;
-  int i;
+  guint i;
   struct StructureItem item;
 
   log ("\n");
@@ -799,7 +815,7 @@ test_create (gconstpointer test_data)
   GFile *root, *child;
   gboolean res;
   GError *error;
-  int i;
+  guint i;
   struct StructureItem item;
   GFileOutputStream *os;
 
@@ -885,7 +901,7 @@ test_open (gconstpointer test_data)
   GFile *root, *child;
   gboolean res;
   GError *error;
-  int i;
+  guint i;
   struct StructureItem item;
   GFileInputStream *input_stream;
 
@@ -956,7 +972,7 @@ test_delete (gconstpointer test_data)
   GFile *child;
   gboolean res;
   GError *error;
-  int i;
+  guint i;
   struct StructureItem item;
 
   g_assert (test_data != NULL);
@@ -1092,7 +1108,7 @@ int
 main (int argc, char *argv[])
 {
   static gboolean only_create_struct;
-  static char *target_path;
+  const char *target_path;
   GError *error;
   GOptionContext *context;
 
@@ -1107,6 +1123,7 @@ main (int argc, char *argv[])
     {NULL}
   };
 
+  test_suite = FALSE;
   verbose = FALSE;
   write_test = FALSE;
   only_create_struct = FALSE;
@@ -1120,6 +1137,7 @@ main (int argc, char *argv[])
   /*  no extra parameters specified, assume we're executed from glib test suite  */ 
   if (argc < 2)
     {
+	  test_suite = TRUE;
 	  verbose = TRUE;
 	  write_test = TRUE;
 	  only_create_struct = FALSE;
