@@ -190,9 +190,10 @@ g_io_unix_read (GIOChannel *channel,
 
   if (result < 0)
     {
+      int errsv = errno;
       *bytes_read = 0;
 
-      switch (errno)
+      switch (errsv)
         {
 #ifdef EINTR
           case EINTR:
@@ -204,8 +205,8 @@ g_io_unix_read (GIOChannel *channel,
 #endif
           default:
             g_set_error_literal (err, G_IO_CHANNEL_ERROR,
-                                 g_io_channel_error_from_errno (errno),
-                                 g_strerror (errno));
+                                 g_io_channel_error_from_errno (errsv),
+                                 g_strerror (errsv));
             return G_IO_STATUS_ERROR;
         }
     }
@@ -230,9 +231,10 @@ g_io_unix_write (GIOChannel  *channel,
 
   if (result < 0)
     {
+      int errsv = errno;
       *bytes_written = 0;
 
-      switch (errno)
+      switch (errsv)
         {
 #ifdef EINTR
           case EINTR:
@@ -244,8 +246,8 @@ g_io_unix_write (GIOChannel  *channel,
 #endif
           default:
             g_set_error_literal (err, G_IO_CHANNEL_ERROR,
-                                 g_io_channel_error_from_errno (errno),
-                                 g_strerror (errno));
+                                 g_io_channel_error_from_errno (errsv),
+                                 g_strerror (errsv));
             return G_IO_STATUS_ERROR;
         }
     }
@@ -295,9 +297,10 @@ g_io_unix_seek (GIOChannel *channel,
 
   if (result < 0)
     {
+      int errsv = errno;
       g_set_error_literal (err, G_IO_CHANNEL_ERROR,
-                           g_io_channel_error_from_errno (errno),
-                           g_strerror (errno));
+                           g_io_channel_error_from_errno (errsv),
+                           g_strerror (errsv));
       return G_IO_STATUS_ERROR;
     }
 
@@ -313,9 +316,10 @@ g_io_unix_close (GIOChannel *channel,
 
   if (close (unix_channel->fd) < 0)
     {
+      int errsv = errno;
       g_set_error_literal (err, G_IO_CHANNEL_ERROR,
-                           g_io_channel_error_from_errno (errno),
-                           g_strerror (errno));
+                           g_io_channel_error_from_errno (errsv),
+                           g_strerror (errsv));
       return G_IO_STATUS_ERROR;
     }
 
@@ -376,9 +380,10 @@ g_io_unix_set_flags (GIOChannel *channel,
 
   if (fcntl (unix_channel->fd, F_SETFL, fcntl_flags) == -1)
     {
+      int errsv = errno;
       g_set_error_literal (err, G_IO_CHANNEL_ERROR,
-                           g_io_channel_error_from_errno (errno),
-                           g_strerror (errno));
+                           g_io_channel_error_from_errno (errsv),
+                           g_strerror (errsv));
       return G_IO_STATUS_ERROR;
     }
 
@@ -396,8 +401,9 @@ g_io_unix_get_flags (GIOChannel *channel)
 
   if (fcntl_flags == -1)
     {
+      int err = errno;
       g_warning (G_STRLOC "Error while getting flags for FD: %s (%d)\n",
-		 g_strerror (errno), errno);
+		 g_strerror (err), err);
       return 0;
     }
 
@@ -512,18 +518,20 @@ g_io_channel_new_file (const gchar *filename,
   fid = open (filename, flags, create_mode);
   if (fid == -1)
     {
+      int err = errno;
       g_set_error_literal (error, G_FILE_ERROR,
-                           g_file_error_from_errno (errno),
-                           g_strerror (errno));
+                           g_file_error_from_errno (err),
+                           g_strerror (err));
       return (GIOChannel *)NULL;
     }
 
   if (fstat (fid, &buffer) == -1) /* In case someone opens a FIFO */
     {
+      int err = errno;
       close (fid);
       g_set_error_literal (error, G_FILE_ERROR,
-                           g_file_error_from_errno (errno),
-                           g_strerror (errno));
+                           g_file_error_from_errno (err),
+                           g_strerror (err));
       return (GIOChannel *)NULL;
     }
 
@@ -561,6 +569,32 @@ g_io_channel_new_file (const gchar *filename,
   return channel;
 }
 
+/**
+ * g_io_channel_unix_new:
+ * @fd: a file descriptor.
+ * @Returns: a new #GIOChannel.
+ *
+ * Creates a new #GIOChannel given a file descriptor. On UNIX systems
+ * this works for plain files, pipes, and sockets.
+ *
+ * The returned #GIOChannel has a reference count of 1.
+ *
+ * The default encoding for #GIOChannel is UTF-8. If your application
+ * is reading output from a command using via pipe, you may need to set
+ * the encoding to the encoding of the current locale (see
+ * g_get_charset()) with the g_io_channel_set_encoding() function.
+ *
+ * If you want to read raw binary data without interpretation, then
+ * call the g_io_channel_set_encoding() function with %NULL for the
+ * encoding argument.
+ *
+ * This function is available in GLib on Windows, too, but you should
+ * avoid using it on Windows. The domain of file descriptors and
+ * sockets overlap. There is no way for GLib to know which one you mean
+ * in case the argument you pass to this function happens to be both a
+ * valid file descriptor and socket. If that happens a warning is
+ * issued, and GLib assumes that it is the file descriptor you mean.
+ **/
 GIOChannel *
 g_io_channel_unix_new (gint fd)
 {
@@ -590,6 +624,16 @@ g_io_channel_unix_new (gint fd)
   return channel;
 }
 
+/**
+ * g_io_channel_unix_get_fd:
+ * @channel: a #GIOChannel, created with g_io_channel_unix_new().
+ * @Returns: the file descriptor of the #GIOChannel.
+ *
+ * Returns the file descriptor of the #GIOChannel.
+ *
+ * On Windows this function returns the file descriptor or socket of
+ * the #GIOChannel.
+ **/
 gint
 g_io_channel_unix_get_fd (GIOChannel *channel)
 {
