@@ -27,7 +27,7 @@
 #undef G_DISABLE_ASSERT
 #undef G_LOG_DOMAIN
 
-#include <config.h>
+#include "config.h"
 
 #include <glib.h>
 #include <stdio.h>
